#################################################################################
# Some sorting algorithms:                                                      #
#  - selection sort                                                             #
#  - quick sort                                                                 #
#  - timsort                                                                    #
# Also within the code tracking time taken for checking algorithm efficiency    #
# Author: Selina Dennis, 2016                                                   #
# Attributions: QuickSort is Magnus Lie Hetland's version                       #
#   found at: http://hetland.org/coding/python/quicksort.html                   #
#################################################################################

import timeit, random

#################################################################################
# Selection Sort
# Required Arguments:
#  - a list of elements to be sorted (can be any data type)
# Return Value:
#  None
#################################################################################
def selectionSort( aList ):
    for i in range( len( aList ) ):
        smallest = i
        for k in range( i+1 , len( aList ) ):
            if aList[k] < aList[smallest]:
                smallest = k
        if smallest != i:
            temp = aList[smallest]
            aList[smallest] = aList[i]
            aList[i] = temp

#################################################################################
# Quick Sort (a hack function that just kicks off the 'real' quicksort)
#  - it was made to interact neatly with timeit, as not having it
#    takes more effort - efficiency ftw!
# Required Arguments:
#  - a list of elements to be sorted 
# Return Value:
#  None
#################################################################################
def quickSort( aList ):
    start = 0
    end = len(aList)-1
    qSort(aList, start, end)

#################################################################################
# Helper Function: Quick Sort
# Required Arguments:
#  - a list of elements to be sorted 
#  - a start position
#  - an end position
# Return Value:
#  None
#################################################################################       
def qSort(aList, start, end):
    if start < end:                          # If two or more elements...
        split = partition(aList, start, end) # ... partition the sublist...
        qSort(aList, start, split-1)         # ... and sort both halves.
        qSort(aList, split, end)
        
#################################################################################
# Partition Function: Quick Sort
# Required Arguments:
#  - a list of elements to be sorted 
#  - a start position
#  - an end position
# Return Value:
#  Integer: the index value where the whole list should be split 
#################################################################################
def partition( aList, start, end ):
    pivot = aList[end]
    bottom = start-1
    top = end
    done = False
    while not done:
        while not done:
            bottom += 1
            if bottom == top:
                done = True
                break
            if aList[bottom] > pivot:
                aList[top] = aList[bottom]
                break
        while not done:
            top -= 1
            if top == bottom:
                done = True
                break
            if aList[top] < pivot:
                aList[bottom] = aList[top]
                break
    aList[top] = pivot
    return top

#################################################################################
# Timsort
# Required Arguments:
#  - a list of elements to be sorted 
# Return Value:
#  None
#################################################################################
def timSort( aList ):
    aList.sort()

# If this script is run as a program:
if __name__=="__main__":
   
    print 45*"\n"

    while True:
        enteredText = raw_input("(type 'exit' to quit)\nStarting number in sample set: n=")

        # check to see if they want to quit.
        if enteredText in ["exit", "quit", "q", "x"]:
            break
        
        # validationg: using "try" to handle any non-integers or control sequences/etc
        # entered by the user
        try:
            # Construct multiple versions of the same randomised list
            # to make sure all of the sorting occurs on the same unsorted list.
            # This allows for accurate representations of how long the algorithms
            # take to run.
            baseList = random.sample(xrange(10000000), int(enteredText))
            A = list(baseList)
            B = list(baseList)
            C = list(baseList)

            # Run the three algorithms and print the time taken using the
            # performance package 'timeit'
            print "\n\nRunning on %d entries..." % len(A)
            print 'Selection sort takes: \t%12.5f seconds' % timeit.timeit("selectionSort(A)", setup = "from __main__ import selectionSort, A", number=1)
            print 'Quick sort takes: \t%12.5f seconds' % timeit.timeit("quickSort(B)", setup = "from __main__ import quickSort, B", number=1)
            print 'Timsort takes: \t\t%12.5f seconds' % timeit.timeit("timSort(C)", setup = "from __main__ import timSort, C", number=1)    
            print 2*"\n"
        # Exception has occurred, check to see if it was a type error
        except ValueError:
            print "Enter whole, positive numbers only."
        # Any other error that's not a type error should abandon the program
        except:
            break           
    
    print "Finished."
